"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.typeDefs = exports.typeDefsString = void 0;
const jwt_1 = require("../jwt");
const client_1 = require("@deep-foundation/hasura/client");
const graphql_tag_1 = __importDefault(require("graphql-tag"));
const apollo_server_express_1 = require("apollo-server-express");
const client_2 = require("../client");
const JWT_SECRET = process.env.JWT_SECRET;
const jwt_secret = JSON.parse(JWT_SECRET);
exports.typeDefsString = `
  type Query {
    guest: GuestOutput
  }
  type GuestOutput {
    token: String
    linkId: Int
  }
`;
exports.typeDefs = (0, graphql_tag_1.default) `${exports.typeDefsString}`;
const client = (0, client_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_2.DeepClient({
    apolloClient: client,
});
const resolvers = {
    Query: {
        guest: (source, args, context, info) => __awaiter(void 0, void 0, void 0, function* () {
            const { data: [{ id }] } = yield deep.insert({ type_id: yield deep.id('@deep-foundation/core', 'User') });
            const token = (0, jwt_1.jwt)({
                secret: jwt_secret.key,
                linkId: id,
            });
            return { token, linkId: id };
        }),
    }
};
const context = ({ req }) => {
    return { headers: req.headers };
};
const apolloServer = new apollo_server_express_1.ApolloServer({ introspection: true, typeDefs: exports.typeDefs, resolvers, context });
exports.default = apolloServer;
//# sourceMappingURL=guest.js.map