"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.api = void 0;
const debug_1 = __importDefault(require("debug"));
const client_1 = require("@deep-foundation/hasura/client");
const api_1 = require("@deep-foundation/hasura/api");
const apollo_boost_1 = require("apollo-boost");
const promise_1 = require("../promise");
const client_2 = require("../client");
const global_ids_1 = require("../global-ids");
const child_process_1 = require("child_process");
const axios_1 = __importDefault(require("axios"));
const crypto_1 = __importDefault(require("crypto"));
const SCHEMA = 'public';
const debug = (0, debug_1.default)('deepcase:eh');
exports.api = new api_1.HasuraApi({
    path: process.env.DEEPLINKS_HASURA_PATH,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const client = (0, client_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_2.DeepClient({
    apolloClient: client,
});
const portHash = {};
const containerHash = {};
const UseRunner = ({ code, link }) => __awaiter(void 0, void 0, void 0, function* () {
    console.log("handler4: ");
    const runnerPort = 3020;
    const runnerImageAndTag = 'menzorg/deep-runner-js:main';
    const jwt = 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJodHRwczovL2hhc3VyYS5pby9qd3QvY2xhaW1zIjp7IngtaGFzdXJhLWFsbG93ZWQtcm9sZXMiOlsibGluayJdLCJ4LWhhc3VyYS1kZWZhdWx0LXJvbGUiOiJsaW5rIiwieC1oYXN1cmEtdXNlci1pZCI6IjM5In0sImlhdCI6MTYzNzAzMjQwNn0.EtYolslSV66xKe7Bx4x3MkS-dQL5hPqaUqE0eStH3KE';
    const data = jwt + code;
    const containerName = crypto_1.default.createHash('md5').update(data).digest("hex");
    let port;
    if (!portHash[containerName]) {
        port = Math.floor(Math.random() * 1000) + 1000;
        portHash[containerName] = port;
        containerHash[port] = containerName;
        const startDocker = `docker run --name ${containerName} -p ${port}:${runnerPort} -d ${runnerImageAndTag} && npx -q wait-on --timeout 20000 http://localhost:${port}/healthz`;
        let startResult;
        try {
            startResult = yield (0, child_process_1.execSync)(startDocker).toString();
        }
        catch (e) {
            startResult = e.stderr;
        }
        console.log('startResult', startResult.toString());
        console.log(1);
        while (startResult.indexOf('port is already arllocated') !== -1) {
            console.log(2);
            containerHash[port] = 'broken';
            port = Math.floor(Math.random() * 1000) + 1000;
            containerHash[port] = containerName;
            const REstartDocker = `docker run --name ${containerName} -p ${port}:3020 -d ${runnerImageAndTag} && npx -q wait-on --timeout 20000 http://localhost:${port}/healthz`;
            const startResult = yield (0, child_process_1.execSync)(REstartDocker).toString();
            console.log('REstartResult', startResult);
        }
        console.log(3);
        if (startResult.indexOf('is already in use by container') !== -1) {
            console.log(4);
            const REstartDocker = `docker stop ${containerName} && docker rm ${containerName} && docker run --name ${containerName} -p ${port}:3020 -d ${runnerImageAndTag} && npx -q wait-on --timeout 10000 http://localhost:${port}/healthz`;
            try {
                yield (0, child_process_1.execSync)(REstartDocker).toString();
            }
            catch (e) {
                console.log(e === null || e === void 0 ? void 0 : e.status);
            }
        }
        console.log(5);
        const initResult = yield axios_1.default.post(`http://localhost:${port}/init`, { params: { code, jwt } });
        console.log('initResult', initResult.data);
        console.log(6);
    }
    else {
        port = portHash[containerName];
    }
    console.log(7);
    try {
        const result = yield axios_1.default.post(`http://localhost:${port}/call`, { params: { link, } });
        return (result.data);
    }
    catch (e) {
        console.log('e', e);
    }
});
exports.default = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    try {
        const event = (_a = req === null || req === void 0 ? void 0 : req.body) === null || _a === void 0 ? void 0 : _a.event;
        const operation = event === null || event === void 0 ? void 0 : event.op;
        if (operation === 'INSERT' || operation === 'UPDATE' || operation === 'DELETE') {
            const oldRow = (_b = event === null || event === void 0 ? void 0 : event.data) === null || _b === void 0 ? void 0 : _b.old;
            const newRow = (_c = event === null || event === void 0 ? void 0 : event.data) === null || _c === void 0 ? void 0 : _c.new;
            const current = operation === 'DELETE' ? oldRow : newRow;
            const typeId = current.type_id;
            console.log('current', current, typeId);
            try {
                const handlerTypeId = yield deep.id('@deep-foundation/core', 'Handler');
                const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
                const queryString = `query SELECT_CODE($typeId: bigint, $linkId: bigint) { links(where: {
          type_id: { _eq: ${yield deep.id('@deep-foundation/core', 'SyncTextFile')} },
          in: {
            from_id: { _eq: ${yield deep.id('@deep-foundation/core', 'JSExecutionProvider')} },
            type_id: { _eq: ${handlerTypeId} },
            in: {
              _or: [
                {
                  from: {
                    type_id: { _eq: $typeId },
                  }
                },
                #{
                #  from: {
                #    type_id: { _eq: ${yield deep.id('@deep-foundation/core', 'Selector')} },
                #    out: {
                #      type_id: { _eq: ${yield deep.id('@deep-foundation/core', 'Selection')} },
                #      to_id: { _eq: $linkId },
                #    }
                #  }
                #}
              ],
              type_id: { _eq: ${handleInsertTypeId} },
            }
          }
        }) {
          id
          value
          in(where: { type_id: { _eq: ${handlerTypeId} } }) {
            id
            in(where: { type_id: { _eq: ${handleInsertTypeId} } }) {
              id
            }
          }
        } }`;
                const query = (0, apollo_boost_1.gql) `${queryString}`;
                const handleStringResult = yield client.query({ query, variables: {
                        typeId,
                        linkId: newRow.id,
                    } });
                const promises = [];
                const handleInsertsIds = [];
                const handlersWithCode = (_d = handleStringResult === null || handleStringResult === void 0 ? void 0 : handleStringResult.data) === null || _d === void 0 ? void 0 : _d.links;
                if ((handlersWithCode === null || handlersWithCode === void 0 ? void 0 : handlersWithCode.length) > 0) {
                    console.log("handlersWithCode: ", JSON.stringify(handlersWithCode, null, 2));
                    console.log(handlersWithCode === null || handlersWithCode === void 0 ? void 0 : handlersWithCode.length);
                    for (const handlerWithCode of handlersWithCode) {
                        const code = (_e = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.value) === null || _e === void 0 ? void 0 : _e.value;
                        const handleInsertId = (_h = (_g = (_f = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.in) === null || _f === void 0 ? void 0 : _f[0]) === null || _g === void 0 ? void 0 : _g.in) === null || _h === void 0 ? void 0 : _h[0].id;
                        if (code) {
                            try {
                                promises.push(UseRunner({ code, link: newRow }));
                                handleInsertsIds.push(handleInsertId);
                            }
                            catch (error) {
                                debug('error', error);
                            }
                        }
                    }
                    const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
                    const rejectedTypeId = yield deep.id('@deep-foundation/core', 'Rejected');
                    const promiseResultTypeId = yield deep.id('@deep-foundation/core', 'PromiseResult');
                    const promiseReasonTypeId = yield deep.id('@deep-foundation/core', 'PromiseReason');
                    const promise = yield (0, promise_1.findPromiseLink)({
                        id: newRow.id, client: deep.apolloClient,
                        Then: yield deep.id('@deep-foundation/core', 'Then'),
                        Promise: yield deep.id('@deep-foundation/core', 'Promise'),
                        Resolved: resolvedTypeId,
                        Rejected: rejectedTypeId,
                    });
                    if (promise) {
                        console.log('promise: ', promise);
                        console.log("promises.length: ", promises.length);
                        Promise.allSettled(promises)
                            .then((values) => __awaiter(void 0, void 0, void 0, function* () {
                            console.log("values: ", values);
                            for (let i = 0; i < values.length; i++) {
                                const value = values[i];
                                const handleInsertId = handleInsertsIds[i];
                                if (value.status == 'fulfilled') {
                                    const result = value.value;
                                    console.log("result: ", result);
                                    let promiseResult = (yield deep.insert({
                                        from_id: 0,
                                        type_id: promiseResultTypeId,
                                        to_id: 0
                                    }, { name: 'IMPORT_PROMISE_RESULT' })).data[0];
                                    yield deep.insert({
                                        link_id: promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.id,
                                        value: result
                                    }, { table: 'objects' });
                                    const resolveLink = (yield deep.insert({
                                        from_id: promise.id,
                                        type_id: resolvedTypeId,
                                        to_id: promiseResult.id
                                    }, { name: 'IMPORT_RESOLVE_LINK' })).data[0];
                                    const promiseReason = (yield deep.insert({
                                        from_id: resolveLink.id,
                                        type_id: promiseReasonTypeId,
                                        to_id: handleInsertId
                                    }, { name: 'IMPORT_PROMISE_REASON' })).data[0];
                                }
                                if (value.status == 'rejected') {
                                    const error = value.reason;
                                    console.log("error: ", error);
                                    let insertedLink = (yield deep.insert({
                                        type_id: promiseResultTypeId,
                                    }, { name: 'IMPORT_PROMISE_RESULT' })).data[0];
                                    yield deep.insert({
                                        link_id: insertedLink === null || insertedLink === void 0 ? void 0 : insertedLink.id,
                                        value: error
                                    }, { table: 'objects' });
                                    insertedLink = (yield deep.insert({
                                        from_id: promise.id,
                                        type_id: rejectedTypeId,
                                        to_id: insertedLink.id
                                    }, { name: 'IMPORT_REJECT_LINK' })).data[0];
                                }
                            }
                        }));
                    }
                }
                if (operation === 'INSERT' && !global_ids_1.DENIED_IDS.includes(current.type_id) && global_ids_1.ALLOWED_IDS.includes(current.type_id)) {
                    debug('resolve', current.id);
                    yield (0, promise_1.resolve)({
                        id: current.id, client,
                        Then: yield deep.id('@deep-foundation/core', 'Then'),
                        Promise: yield deep.id('@deep-foundation/core', 'Promise'),
                        Resolved: yield deep.id('@deep-foundation/core', 'Resolved'),
                        Rejected: yield deep.id('@deep-foundation/core', 'Rejected'),
                    });
                }
                return res.status(200).json({});
            }
            catch (error) {
                debug('error', error);
                if (operation === 'INSERT' && !global_ids_1.DENIED_IDS.includes(current.type_id) && global_ids_1.ALLOWED_IDS.includes(current.type_id)) {
                    debug('reject', current.id);
                    yield (0, promise_1.reject)({
                        id: current.id, client,
                        Then: yield deep.id('@deep-foundation/core', 'Then'),
                        Promise: yield deep.id('@deep-foundation/core', 'Promise'),
                        Resolved: yield deep.id('@deep-foundation/core', 'Resolved'),
                        Rejected: yield deep.id('@deep-foundation/core', 'Rejected'),
                    });
                }
            }
            return res.status(500).json({ error: 'notexplained' });
        }
        return res.status(500).json({ error: 'operation can be only INSERT or UPDATE' });
    }
    catch (error) {
        return res.status(500).json({ error: error.toString() });
    }
});
//# sourceMappingURL=links.js.map