"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_tag_1 = __importDefault(require("graphql-tag"));
const gists_1 = __importDefault(require("gists"));
const client_1 = require("@deep-foundation/hasura/client");
const client_2 = require("../client");
const packager_1 = require("../packager");
const apollo_server_express_1 = require("apollo-server-express");
const client_3 = require("../client");
const fs_1 = __importDefault(require("fs"));
const os_1 = __importDefault(require("os"));
const uuid_1 = require("uuid");
const child_process_1 = __importDefault(require("child_process"));
const semver = __importStar(require("semver"));
const tmpdir = os_1.default.tmpdir();
const gists = new gists_1.default({
    username: 'ivansglazunov',
    password: (_a = process === null || process === void 0 ? void 0 : process.env) === null || _a === void 0 ? void 0 : _a.PASSWORD,
});
const client = (0, client_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_3.DeepClient({ apolloClient: client });
const packager = new packager_1.Packager(deep);
const typeDefs = (0, graphql_tag_1.default) `
  type Query {
    packagerInstall(input: PackagerInstallInput): PackagerInstallOutput
    packagerPublish(input: PackagerPublishInput): PackagerPublishOutput
  }
  input PackagerInstallInput {
    name: String
    version: String
    address: String
    type: String
  }
  type PackagerInstallOutput {
    ids: [Int]
    errors: [String]
  }
  input PackagerPublishInput {
    id: Int
    address: String
    type: String
  }
  type PackagerPublishOutput {
    ids: [Int]
    errors: [String]
  }
`;
const resolvers = {
    Query: {
        packagerInstall: (source, args, context, info) => __awaiter(void 0, void 0, void 0, function* () {
            var _b, _c;
            const { name, version, address, type } = args.input;
            const errors = [];
            if (type === 'gist') {
                const result = yield gists.get(address);
                const files = (_b = result === null || result === void 0 ? void 0 : result.body) === null || _b === void 0 ? void 0 : _b.files;
                const deepPckgContent = (_c = files === null || files === void 0 ? void 0 : files['deep.json']) === null || _c === void 0 ? void 0 : _c.content;
                if (deepPckgContent) {
                    try {
                        const deepPckg = JSON.parse(deepPckgContent);
                        const { ids, errors } = yield packager.import(deepPckg);
                        return { ids, errors };
                    }
                    catch (error) {
                        errors.push(error);
                    }
                }
                else {
                    errors.push(`deep.json not founded in gist`);
                }
            }
            else if (type === 'npm') {
                const dirid = (0, uuid_1.v4)();
                const dir = [tmpdir, dirid].join('/');
                fs_1.default.mkdirSync(dir);
                let selector = address;
                if (version)
                    selector += '@' + version;
                try {
                    child_process_1.default.execSync(`cd ${dir}; npm install --no-cache ${selector};`, { stdio: [0, 1, 2] });
                }
                catch (error) {
                    errors.push(error);
                    errors.push('istallation failed');
                    return { errors };
                }
                const npmPckgContent = fs_1.default.readFileSync([dir, 'node_modules', address, 'package.json'].join('/'), { encoding: 'utf-8' });
                const deepPckgContent = fs_1.default.readFileSync([dir, 'node_modules', address, 'deep.json'].join('/'), { encoding: 'utf-8' });
                if (deepPckgContent) {
                    try {
                        const npmPckg = JSON.parse(npmPckgContent);
                        const deepPckg = JSON.parse(deepPckgContent);
                        const { ids, errors } = yield packager.import(deepPckg);
                        fs_1.default.rmSync(dir, { recursive: true, force: true });
                        return { ids, errors };
                    }
                    catch (error) {
                        errors.push(error);
                    }
                }
                else {
                    errors.push(`deep.json not founded in gist`);
                }
                fs_1.default.rmSync(dir, { recursive: true, force: true });
            }
            else {
                errors.push(`type ${type} is not valid`);
            }
            return { ids: [], errors };
        }),
        packagerPublish: (source, args, context, info) => __awaiter(void 0, void 0, void 0, function* () {
            const { id, address, type } = args.input;
            const errors = [];
            if (type === 'gist') {
            }
            else if (type === 'npm') {
                const dirid = (0, uuid_1.v4)();
                const dir = [tmpdir, dirid].join('/');
                const pckgDir = [tmpdir, dirid, 'node_modules', address].join('/');
                fs_1.default.mkdirSync(dir);
                try {
                    child_process_1.default.execSync(`cd ${dir}; npm install --no-cache ${address};`, { stdio: [0, 1, 2] });
                }
                catch (error) {
                    errors.push(error);
                    errors.push('installation failed');
                    return { errors };
                }
                const npmPckgPath = [pckgDir, 'package.json'].join('/');
                const npmPckgJSON = fs_1.default.readFileSync(npmPckgPath, { encoding: 'utf-8' });
                let npmPckg;
                let nextVersion;
                if (!npmPckgJSON) {
                    errors.push('package.json not founded in installed package');
                    return { errors };
                }
                else {
                    npmPckg = JSON.parse(npmPckgJSON);
                    npmPckg.version = nextVersion = semver.inc((npmPckg === null || npmPckg === void 0 ? void 0 : npmPckg.version) || '0.0.0', 'patch');
                }
                yield deep.update({
                    link: {
                        type_id: { _eq: client_2.GLOBAL_ID_PACKAGE_VERSION },
                        to_id: { _eq: id },
                    },
                }, { value: nextVersion }, { table: 'strings' });
                fs_1.default.writeFileSync(npmPckgPath, JSON.stringify(npmPckg), { encoding: 'utf-8' });
                const deepPckgContent = yield packager.export({ packageLinkId: id });
                console.log(JSON.stringify(deepPckgContent, null, 2));
                fs_1.default.writeFileSync([pckgDir, 'deep.json'].join('/'), JSON.stringify(deepPckgContent), { encoding: 'utf-8' });
                child_process_1.default.execSync(`cd ${pckgDir}; npm publish;`, { stdio: [0, 1, 2] });
                fs_1.default.rmSync(dir, { recursive: true, force: true });
            }
            return { errors };
        }),
    }
};
const context = ({ req }) => {
    return { headers: req.headers };
};
const apolloServer = new apollo_server_express_1.ApolloServer({ introspection: true, typeDefs, resolvers, context });
exports.default = apolloServer;
//# sourceMappingURL=packager.js.map