"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const client_1 = require("@deep-foundation/hasura/client");
const client_2 = require("../imports/client");
const chai_1 = require("chai");
const graphql_tag_1 = __importDefault(require("graphql-tag"));
const apolloClient = (0, client_1.generateApolloClient)({
    path: `${process.env.HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.HASURA_SSL,
    secret: process.env.HASURA_SECRET,
});
const deep = new client_2.DeepClient({ apolloClient });
const DELAY = +process.env.DELAY || 0;
const delay = time => new Promise(res => setTimeout(res, time));
const insertHandlerForType = (typeId, code) => __awaiter(void 0, void 0, void 0, function* () {
    const syncTextFileTypeId = yield deep.id('@deep-foundation/core', 'SyncTextFile');
    const handlerTypeId = yield deep.id('@deep-foundation/core', 'Handler');
    const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
    const jsExecutionProviderId = yield deep.id('@deep-foundation/core', 'JSExecutionProvider');
    let handlerJSFile = (yield deep.insert({
        type_id: syncTextFileTypeId
    }, { name: 'IMPORT_HANDLER_JS_FILE' })).data[0];
    let handlerJSFileValue = (yield deep.insert({ link_id: handlerJSFile === null || handlerJSFile === void 0 ? void 0 : handlerJSFile.id, value: code }, { table: 'strings' })).data[0];
    const handler = (yield deep.insert({
        from_id: jsExecutionProviderId,
        type_id: handlerTypeId,
        to_id: handlerJSFile === null || handlerJSFile === void 0 ? void 0 : handlerJSFile.id,
    }, { name: 'IMPORT_HANDLER' })).data[0];
    const handleInsert = (yield deep.insert({
        from_id: typeId,
        type_id: handleInsertTypeId,
        to_id: handler === null || handler === void 0 ? void 0 : handler.id,
    }, { name: 'IMPORT_INSERT_HANDLER' })).data[0];
    return {
        handlerId: handler === null || handler === void 0 ? void 0 : handler.id,
        handleInsertId: handleInsert === null || handleInsert === void 0 ? void 0 : handleInsert.id,
        handlerJSFileId: handlerJSFile === null || handlerJSFile === void 0 ? void 0 : handlerJSFile.id,
        handlerJSFileValueId: handlerJSFileValue === null || handlerJSFileValue === void 0 ? void 0 : handlerJSFileValue.id,
    };
});
const deleteHandler = (handler) => __awaiter(void 0, void 0, void 0, function* () {
    yield deep.delete({ id: { _in: [handler.handlerJSFileId, handler.handlerId, handler.handleInsertId] } });
    yield deep.delete({ id: { _eq: handler.handlerJSFileValueId } }, { table: 'strings' });
});
jest.setTimeout(60000);
function randomInteger(min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
}
describe('sync function handle by type with resolve', () => {
    it(`handle insert`, () => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w;
        const numberToReturn = randomInteger(5000000, 9999999999);
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        var handler = yield insertHandlerForType(typeId, `(arg)=>{console.log(arg); return {result: ${numberToReturn}}}`);
        const freeId = randomInteger(5000000, 9999999999);
        console.log(freeId);
        const promiseTypeId = yield deep.id('@deep-foundation/core', 'Promise');
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const thenTypeId = yield deep.id('@deep-foundation/core', 'Then');
        const linkInsert = (yield deep.insert({
            id: freeId,
            from_id: freeId,
            type_id: typeId,
            to_id: freeId
        }, { name: 'IMPORT_LINK' })).data[0];
        console.log(linkInsert);
        chai_1.assert.equal(freeId, linkInsert.id);
        yield deep.await(freeId);
        const client = deep.apolloClient;
        const resultLinks = (_b = (_a = (yield client.query({
            query: (0, graphql_tag_1.default) `{
        links(where: { 
          in: {
            type_id: { _eq: ${resolvedTypeId} }, # Resolved
            from: { 
              type_id: { _eq: ${promiseTypeId} }, # Promise
              in: { 
                type_id: { _eq: ${thenTypeId} } # Then
                from_id: { _eq: ${freeId} } # freeId
              }
            }
          },
        }) {
          id
          object {
            id
            value
          }
          in(where: { type_id: { _eq: ${resolvedTypeId} } }) {
            id
            from {
              id
              in(where: { type_id: { _eq: ${thenTypeId} } }) {
                id
              }
            }
          }
        }
      }`,
        }))) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.links;
        const resolvedLinkId = (_e = (_d = (_c = resultLinks === null || resultLinks === void 0 ? void 0 : resultLinks[0]) === null || _c === void 0 ? void 0 : _c.in) === null || _d === void 0 ? void 0 : _d[0]) === null || _e === void 0 ? void 0 : _e.id;
        const thenLinkId = (_l = (_k = (_j = (_h = (_g = (_f = resultLinks === null || resultLinks === void 0 ? void 0 : resultLinks[0]) === null || _f === void 0 ? void 0 : _f.in) === null || _g === void 0 ? void 0 : _g[0]) === null || _h === void 0 ? void 0 : _h.from) === null || _j === void 0 ? void 0 : _j.in) === null || _k === void 0 ? void 0 : _k[0]) === null || _l === void 0 ? void 0 : _l.id;
        const valueId = (_o = (_m = resultLinks === null || resultLinks === void 0 ? void 0 : resultLinks[0]) === null || _m === void 0 ? void 0 : _m.object) === null || _o === void 0 ? void 0 : _o.id;
        const promiseResultId = (_p = resultLinks === null || resultLinks === void 0 ? void 0 : resultLinks[0]) === null || _p === void 0 ? void 0 : _p.id;
        const promiseId = (_t = (_s = (_r = (_q = resultLinks === null || resultLinks === void 0 ? void 0 : resultLinks[0]) === null || _q === void 0 ? void 0 : _q.in) === null || _r === void 0 ? void 0 : _r[0]) === null || _s === void 0 ? void 0 : _s.from) === null || _t === void 0 ? void 0 : _t.id;
        chai_1.assert.equal((_w = (_v = (_u = resultLinks[0]) === null || _u === void 0 ? void 0 : _u.object) === null || _v === void 0 ? void 0 : _v.value) === null || _w === void 0 ? void 0 : _w.result, numberToReturn);
        yield deep.delete({ id: { _in: [resolvedLinkId, thenLinkId] } }, { table: 'links' });
        yield deep.delete({ id: { _in: [promiseResultId, promiseId, freeId] } }, { table: 'links' });
        yield deep.delete({ id: { _eq: valueId } }, { table: 'objects' });
        yield deleteHandler(handler);
    }));
});
const itDelay = () => {
    if (DELAY) {
        (it)('delay', () => __awaiter(void 0, void 0, void 0, function* () {
            yield delay(DELAY);
        }));
    }
};
//# sourceMappingURL=handlers.js.map